/*
 *    smith.H --- Wrapper for computing the smith normal form using pari
 *                (pari function "ZM_snf").
 *                      Part of Foamho, a sl3-homology calculator.
 *
 * Copyright (C) 2012/2013 Lukas Lewark <lewark@math.jussieu.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */

/* Computes integral homology of the given chain complex, using the pari
 * function ZM_snf.
 *
 * Input: CC, ccDims (ranks of the chain complex' modules), ccSpacesSize,
 *    QIdx (number of different q-degrees), minQ (smallest q-degree)
 * Output: torsion, ranks
 */
void smithNormalForm(chainComplex *CC, std::vector<torstruct> &torsion,
        std::vector<std::vector<unsigned> > &ranks,
        std::vector<std::vector<unsigned> > ccDims,
        std::vector<std::vector<unsigned> > IdxTranslator,
        unsigned ccSpacesSize, unsigned QIdx, unsigned minQ);

/* Returns true if compiled with smith.C, false if compiled with
 * smith_without.C.
 */
bool withPari();
